/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOUSE. See the GNU
 * General Public License for more details.
 *
 * You should have recieved a copy of the GNU General Public License
 * along with this program; if not write to the Free Software
 * Foundation, inc., 59 Temple Place, Suite 330, Boston MA 02111-1307
 * USA
 */


/**
 *
 * @author Yuriy Mikhaylovskiy
 * @email YuriyMikhaylovskiy@yahoo.com
 * 
 */


import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.util.*;
import java.io.*;

public class _control_scale_h extends JMCADObject{
  public JSlider slider = new JSlider();
  private JTextField tf0 = new JTextField();
  private JTextField tf1 = new JTextField();
  private double k = 1;
  private double from = 0;
  private double to = 100;

  public void write(RandomAccessFile fout)  throws IOException{
    super.write(fout);
    fout.writeBytes("min = "+from+"\n");
    fout.writeBytes("max = "+to+"\n");
    fout.writeBytes("value = "+slider.getValue()+"\n");
  }

  public void parse(String pst){
    super.parse(pst);

    StringTokenizer stt = new StringTokenizer(pst,"\n");
    while(stt.hasMoreTokens()){
      try{
        String st = stt.nextToken();
        st=st.trim();
        if(st.length()!=0){
          StringTokenizer stt0 = new StringTokenizer(st," =,\t");
          String st0 = stt0.nextToken();
          if(st0.equalsIgnoreCase("MIN")){
            st0 = stt0.nextToken();
            if(!st0.equalsIgnoreCase("NAN"))  try{ from = Double.parseDouble(st0); } catch(Exception e3) {error(e3);}
          }else if(st0.equalsIgnoreCase("MAX")){
            st0 = stt0.nextToken();
            if(!st0.equalsIgnoreCase("NAN"))  try{ to = Double.parseDouble(st0); } catch(Exception e3) {error(e3);}
          }else if(st0.equalsIgnoreCase("VALUE")){
            st0 = stt0.nextToken();
            if(!st0.equalsIgnoreCase("NAN"))  try{ slider.setValue(Integer.parseInt(st0)); } catch(Exception e3) {error(e3);}
          }
        }
      } catch(Exception e1) {error(e1);}
    }
//    try{ init(); } catch(Exception e2) {error(e2);}

    panel_c.removeAll();
    panel_c.add(slider,BorderLayout.CENTER);
  }

  public _control_scale_h() {
    this.isVisual = true;
    ToolTipText = ResourceBundle.getBundle("JMCAD_Internationalize").getString("Scaling_of_entrance_value");
    in = new double[1];
    in_text = new String[1];
    in[0] = Double.NaN;
    in_text[0] = "IN";
    out = new double[1];
    out[0] = Double.NaN;
    out_text = new String[1];
    out_text[0] = "OUT";
    out_obj = new Object[1];
    out_obj_index = new int[1];
    w = 250;
    h = 36;
    slider.addChangeListener(new ChangeListener() {
        public void stateChanged(ChangeEvent e) {
            scale();
        }
    });
    slider.setToolTipText(ToolTipText);
    slider.setMinimum(0);
    slider.setMaximum(100);
    slider.setValue(50);
    slider.setMinimumSize(new Dimension(50,36));
    slider.setMajorTickSpacing(20);
    slider.setMinorTickSpacing(5);
    slider.setPaintTicks(true);
    slider.setPaintLabels(false);
    try { init(); }catch(Exception e) { e.printStackTrace(); }
    panel_c.removeAll();    
    panel_c.add(slider,BorderLayout.CENTER);
  }

  public void calc_pre(){
      double dl = (to - from)/100;
      k = slider.getValue();
      k = from + k * dl;
  }

  public void calc(){
    out[0]=in[0]*k;
    in[0]=Double.NaN;
  }
  
  private void scale(){
      double dl = (to - from)/100;
      k = slider.getValue();
      k = from + k * dl;
  }

  public void paint_info(Graphics g){ }

  public void edit_pre(){
    edit_panel = new JPanel(new VerticalFlowLayout());
    tf0.setText(""+from);
    tf1.setText(""+to);
    JPanel p0 = new JPanel(new BorderLayout());
    JPanel p1 = new JPanel(new BorderLayout());
    p0.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Minimum_value")),BorderLayout.WEST);
    p0.add(tf0,BorderLayout.CENTER);
    p1.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Maximum_value")),BorderLayout.WEST);
    p1.add(tf1,BorderLayout.CENTER);
    edit_panel.add(p0);
    edit_panel.add(p1);
  }

  public void edit_post(){
    try {
      double dfrom = Double.parseDouble(tf0.getText());
      double dto = Double.parseDouble(tf1.getText());
      from = dfrom;
      to = dto;
    }catch(Exception e) { error(e); }
  }

}


