/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOUSE. See the GNU
 * General Public License for more details.
 *
 * You should have recieved a copy of the GNU General Public License
 * along with this program; if not write to the Free Software
 * Foundation, inc., 59 Temple Place, Suite 330, Boston MA 02111-1307
 * USA
 */


/**
 *
 * @author Yuriy Mikhaylovskiy
 * @email YuriyMikhaylovskiy@yahoo.com
 * 
 */


import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.io.*;
import javax.swing.table.*;

public class _graph1 extends JMCADObject {
  private BufferedImage offscreen;
  private BufferedImage offscreen_grid;
  private Graphics2D  offgraphics;
  private Graphics2D  offgraphics_grid;
  private Vector valueX;
  private Vector valueY;
  private double sx,sy,min_x,max_x,min_y,max_y,dx,dy,count,count_i;
  private int w_c,h_c;
  private boolean isPainted = false, isRt = false, isTable = false;
  private JTextField tfw = new JTextField();
  private JTextField tfh = new JTextField();
  private JCheckBox cb = new JCheckBox(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Display_of_data_in_real_time"),false);
  private JCheckBox cb_table = new JCheckBox(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Display_of_data_in_the_form_of_the_table"),false);
  private TableModel tableModel;
  private String[] table_hdr;
  private JTable table;

  public void write(RandomAccessFile fout)  throws IOException{
    super.write(fout);
    fout.writeBytes("width = "+w_c+"\n");
    fout.writeBytes("height = "+h_c+"\n");
    fout.writeBytes("real_time = "+cb.isSelected()+"\n");
    fout.writeBytes("show_table = "+cb_table.isSelected()+"\n");
  }
  public void parse(String pst){
    super.parse(pst);
    StringTokenizer stt = new StringTokenizer(pst,"\n");
    while(stt.hasMoreTokens()){
      try{
        String st = stt.nextToken();
        st=st.trim();
        if(st.length()!=0){
          StringTokenizer stt0 = new StringTokenizer(st," =,\t");
          String st0 = stt0.nextToken();
          if(st0.equalsIgnoreCase("WIDTH")){
            st0 = stt0.nextToken();
            try{ w = Integer.parseInt(st0)+40; } catch(Exception e3) {error(e3);}
            w_c = w-40;
          }else if(st0.equalsIgnoreCase("HEIGHT")){
            st0 = stt0.nextToken();
            try{ h = Integer.parseInt(st0); } catch(Exception e3) {error(e3);}
            h_c = h;
          }else if(st0.equalsIgnoreCase("REAL_TIME")){
            st0 = stt0.nextToken();
            try{ isRt = Boolean.valueOf(st0).booleanValue(); } catch(Exception e3) {error(e3);}
            cb.setSelected(isRt);
          }else if(st0.equalsIgnoreCase("SHOW_TABLE")){
            st0 = stt0.nextToken();
            try{ isTable = Boolean.valueOf(st0).booleanValue(); } catch(Exception e3) {error(e3);}
            cb_table.setSelected(isTable);
          }
        }
      } catch(Exception e1) {error(e1);}
    }
    try{ init(); } catch(Exception e2) {error(e2);}
  }

  public _graph1() {
    this.isVisual = true;
    isGeneranor = true;
    ToolTipText = ResourceBundle.getBundle("JMCAD_Internationalize").getString("The_time_graph:_Y_-_a_variable;_X_-_time_ti");
    in = new double[1];
    in[0] = Double.NaN;
    in_text = new String[1];
    in_text[0] = "IN";
    out = new double[1];
    out[0] = Double.NaN;
    out_text = new String[1];
    out_text[0] = "OUT";
    out_obj = new Object[1];
    out_obj_index = new int[1];
    w = 150;
    h = 100;
    w_c = 150;
    h_c = 100;
    try { init(); }catch(Exception e) { e.printStackTrace(); }
  }
  
  public void paint_info(Graphics g){
    isPainted = true;
    if (offscreen != null) g.drawImage(offscreen, 0, 0,this);
    else g.drawImage(getImageGrid(), 0, 0,this);
    isPainted = false;
  }

  private BufferedImage getImageGrid(){
      offscreen_grid = new BufferedImage(w_c, h_c,BufferedImage.TYPE_3BYTE_BGR);
      offgraphics_grid = (Graphics2D)offscreen_grid.getGraphics();
      //offgraphics_grid.clearRect(0,0,w_c,h_c);
      offgraphics_grid.setColor(this.panel_c.getBackground());
      offgraphics_grid.fillRect(0, 0, w_c, h_c);
      offgraphics_grid.setColor(Color.gray);
      offgraphics_grid.setStroke(new BasicStroke(1));
      double w_step = w_c/10;
      for(double i=0; i<w_c; i+=w_step){
          offgraphics_grid.draw(new Line2D.Double(i,0,i,h_c));
      }
      double h_step = h_c/10;
      for(double i=0; i<h_c; i+=h_step){
          offgraphics_grid.draw(new Line2D.Double(0,i,w_c,i));
      }
      return offscreen_grid;
  }

  public void calc_pre(){
    valueX = new Vector();
    valueY = new Vector();
    if(panel_c.getWidth()!=0)w_c=panel_c.getWidth();
    if(panel_c.getHeight()!=0)h_c=panel_c.getHeight();
    count_i=0;
    count/=w_c;
    sx=1;sy=1;min_x=Double.MAX_VALUE;max_x=Double.MIN_VALUE;min_y=Double.MAX_VALUE;max_y=Double.MIN_VALUE;
    valueX.removeAllElements();
    valueY.removeAllElements();
    offscreen = (BufferedImage)panel_c.createImage(w_c, h_c);
    offgraphics = (Graphics2D)offscreen.getGraphics();
    if(isTable){
      if(valueX==null || valueY==null) return;
      panel_c.removeAll();
      table_hdr = new String[2];
      table_hdr[0]="ti";
      table_hdr[1]="Value";
      tableModel = new AbstractTableModel() {
        public int getColumnCount() { return 2; }
        public int getRowCount() { return valueX.size(); }
        public Object getValueAt(int row, int col){ if(col==0) return valueX.elementAt(row); else return valueY.elementAt(row);}
        public String getColumnName(int col) {return table_hdr[col];}
        public boolean isCellEditable(int row, int col) {return false;}
      };
      table = new JTable(tableModel);
      JScrollPane sp = new JScrollPane();
      sp.getViewport().add(table, null);
      panel_c.setLayout(new BorderLayout());
      panel_c.add(sp, BorderLayout.CENTER);
      this.revalidate();
    }
  }
  public void calc(){
    out[0]=in[0];
    if(Double.isInfinite(in[0]))return;
    valueY.addElement(new Double(in[0]));
    min_y=Math.min(min_y,in[0]);
    max_y=Math.max(max_y,in[0]);
    sy=(max_y-min_y)/h_c;
    dy=-min_y/sy;
    in[0]=Double.NaN;
    if(!isRt)return;
    if(isTable && valueX!=null && valueY!=null){ table.revalidate(); return;}
    if(count<count_i){
      if(isPainted)return;
      offgraphics.drawImage(offscreen_grid, null, 0, 0);
      offgraphics.setColor(Color.BLUE);
      offgraphics.setStroke(new BasicStroke(2));
      double[] vx = new double[valueX.size()];
      double[] vy = new double[valueY.size()];
      for(int i=0; i<valueX.size() && i<valueY.size(); i++){
        vx[i]=((Double)valueX.elementAt(i)).doubleValue()/sx+dx;
        vy[i]=h_c-((Double)valueY.elementAt(i)).doubleValue()/sy-dy;
      }
      try{
        for(int i=0; i<vx.length-1; i++){
          offgraphics.draw(new Line2D.Double(vx[i],vy[i], vx[i+1],vy[i+1]));
        }
      }catch(Exception e) { }
      repaint();
      count_i=0;
    }else count_i++;
    in[0]=Double.NaN;
  }

  public void calc_post(){
    if(isRt)return;
    if(isTable && valueX!=null && valueY!=null){ table.revalidate(); return;}
    offgraphics.drawImage(offscreen_grid, null, 0, 0);
    offgraphics.setColor(Color.BLUE);
    offgraphics.setStroke(new BasicStroke(2));
    if(valueX==null || valueY==null) return;
    double[] vx = new double[valueX.size()];
    double[] vy = new double[valueY.size()];
    for(int i=0; i<valueX.size() && i<valueY.size(); i++){
      vx[i]=((Double)valueX.elementAt(i)).doubleValue()/sx+dx;
      vy[i]=h_c-((Double)valueY.elementAt(i)).doubleValue()/sy-dy;
    }
    try{
      for(int i=0; i<vx.length-1; i++){
        offgraphics.draw(new Line2D.Double(vx[i],vy[i], vx[i+1],vy[i+1]));
      }
    }catch(Exception e) { }
    repaint();
  }

  public synchronized void start(long t0, long ti, long dt){
    valueX.addElement(new Double(ti));
    min_x=t0;
    max_x=ti;
    count=(t0-ti)/h;
    //count = w_c;
    sx=(max_x-min_x)/w_c;
    dx=-min_x/sx;
  }
  public void edit_pre(){
    edit_panel = new JPanel(new VerticalFlowLayout());
    tfw.setText(""+w_c);
    tfh.setText(""+h_c);
    cb.setSelected(isRt);
    cb_table.setSelected(isTable);
    JPanel p0 = new JPanel(new BorderLayout());
    JPanel p1 = new JPanel(new BorderLayout());
    p0.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Width")),BorderLayout.WEST);
    p0.add(tfw,BorderLayout.CENTER);
    p1.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Height")),BorderLayout.WEST);
    p1.add(tfh,BorderLayout.CENTER);
    edit_panel.add(p0);
    edit_panel.add(p1);
    edit_panel.add(cb);
    edit_panel.add(cb_table);
  }
  public void edit_post(){
    try {
      setSize(Integer.parseInt(tfw.getText())+40,Integer.parseInt(tfh.getText()));
      validateTree();
      revalidate();
      w_c=panel_c.getWidth();
      h_c=panel_c.getHeight();
      size_visual_w=w_c;
      size_visual_h=h_c;
      sx=(max_x-min_x)/w_c;
      dx=-min_x/sx;
      sy=(max_y-min_y)/h_c;
      dy=-min_y/sy;
      offscreen = (BufferedImage)panel_c.createImage(w_c, h_c);
      offgraphics = (Graphics2D)offscreen.getGraphics();
      isRt=false;
      calc_post();
      panel_c.removeAll();
      isRt=cb.isSelected();
      isTable=cb_table.isSelected();
      if(isTable){
        if(valueX==null || valueY==null) return;
        panel_c.removeAll();
        table_hdr = new String[2];
        table_hdr[0]="ti";
        table_hdr[1]="Value";
        tableModel = new AbstractTableModel() {
          public int getColumnCount() { return 2; }
          public int getRowCount() { return valueY.size(); }
          public Object getValueAt(int row, int col){ if(col==0) return valueX.elementAt(row); else return valueY.elementAt(row);}
          public String getColumnName(int col) {return table_hdr[col];}
          public boolean isCellEditable(int row, int col) {return false;}
        };
        table = new JTable(tableModel);
        JScrollPane sp = new JScrollPane();
        sp.getViewport().add(table, null);
        panel_c.setLayout(new BorderLayout());
        panel_c.add(sp, BorderLayout.CENTER);
      }
      validateTree();
    }catch(Exception e) { error(e); }
  }

}
