/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOUSE. See the GNU
 * General Public License for more details.
 *
 * You should have recieved a copy of the GNU General Public License
 * along with this program; if not write to the Free Software
 * Foundation, inc., 59 Temple Place, Suite 330, Boston MA 02111-1307
 * USA
 */


/**
 *
 * @author Yuriy Mikhaylovskiy
 * @email YuriyMikhaylovskiy@yahoo.com
 * 
 */


import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.io.*;

public class _oscillograph_multi extends JMCADObject {
  private BufferedImage offscreen;
  private BufferedImage offscreen_grid;
  private Graphics2D  offgraphics;
  private Graphics2D  offgraphics_grid;
  private Vector[] valueY;
  private double sy,min_y,max_y,dy;
  private int w_c,h_c;
  private boolean isPainted = false;
  String[] st_inputs = { "2", "3", "4", "5", "6", "7", "8", "9" };
  JComboBox list_inputs = new JComboBox(st_inputs);
  private JTextField tfw = new JTextField();
  private JTextField tfh = new JTextField();

  public void write(RandomAccessFile fout)  throws IOException{
    super.write(fout);
    fout.writeBytes("inputs = "+in.length+"\n");
    fout.writeBytes("width = "+w_c+"\n");
    fout.writeBytes("height = "+h_c+"\n");
  }

  public void parse(String pst){
    //super.parse(pst);
    StringTokenizer stt = new StringTokenizer(pst,"\n");
    while(stt.hasMoreTokens()){
      try{
        String st = stt.nextToken();
        st=st.trim();
        if(st.length()!=0){
          StringTokenizer stt0 = new StringTokenizer(st," =,\t");
          String st0 = stt0.nextToken();
          if(st0.equalsIgnoreCase("WIDTH")){
            st0 = stt0.nextToken();
            try{ w = Integer.parseInt(st0)+40; } catch(Exception e3) {error(e3);}
            w_c = w-40;
          }else if(st0.equalsIgnoreCase("HEIGHT")){
            st0 = stt0.nextToken();
            try{ h = Integer.parseInt(st0); } catch(Exception e3) {error(e3);}
            h_c = h;
          }else if(st0.equalsIgnoreCase("INPUTS")){
            st0 = stt0.nextToken();
            try{ in = new double[Integer.parseInt(st0)]; } catch(Exception e3) {error(e3);}
          }
        }
      } catch(Exception e1) {error(e1);}
    }
    in_text = new String[in.length];
    out = new double[in.length];
    out_text = new String[in.length];
    out_obj = new Object[in.length];
    out_obj_index = new int[in.length];
    for(int i=0; i<in.length; i++){
        in[i] = Double.NaN;
        in_text[i] = "IN"+i;
        out[i] = Double.NaN;
        out_text[i] = "OUT"+i;
    }
    super.parse(pst);
    try{ init(); } catch(Exception e2) {error(e2);}
  }

  public _oscillograph_multi() {
    this.isVisual = true;
    ToolTipText = ResourceBundle.getBundle("JMCAD_Internationalize").getString("Display_of_data_in_real_time");
    in = new double[2];
    in[0] = Double.NaN;
    in[1] = Double.NaN;
    in_text = new String[2];
    in_text[0] = "IN0";
    in_text[1] = "IN1";
    out = new double[2];
    out[0] = Double.NaN;
    out[1] = Double.NaN;
    out_text = new String[2];
    out_text[0] = "OUT0";
    out_text[1] = "OUT1";
    out_obj = new Object[2];
    out_obj_index = new int[2];
    w = 150;
    h = 100;
    w_c = 150;
    h_c = 100;
    try { init(); }catch(Exception e) { e.printStackTrace(); }
  }
  
  public void paint_info(Graphics g){
    isPainted = true;
    if (offscreen != null) g.drawImage(offscreen, 0, 0,this);
    else g.drawImage(getImageGrid(), 0, 0,this);
    isPainted = false;
  }

  private BufferedImage getImageGrid(){
      if(panel_c.getWidth()!=0)w_c=panel_c.getWidth();
      if(panel_c.getHeight()!=0)h_c=panel_c.getHeight();
      offscreen_grid = new BufferedImage(w_c, h_c,BufferedImage.TYPE_3BYTE_BGR);
      offgraphics_grid = (Graphics2D)offscreen_grid.getGraphics();
      offgraphics_grid.clearRect(0,0,w_c,h_c);
      offgraphics_grid.setColor(Color.gray);
      offgraphics_grid.setStroke(new BasicStroke(1));
      double w_step = w_c/10;
      for(double i=0; i<w_c; i+=w_step){
          offgraphics_grid.draw(new Line2D.Double(i,0,i,h_c));
      }
      double h_step = h_c/10;
      for(double i=0; i<h_c; i+=h_step){
          offgraphics_grid.draw(new Line2D.Double(0,i,w_c,i));
      }
      return offscreen_grid;
  }

  public void calc_pre(){
    valueY = new Vector[in.length];
    for(int i=0; i<valueY.length; i++)valueY[i] = new Vector();
    if(panel_c.getWidth()!=0)w_c=panel_c.getWidth();
    if(panel_c.getHeight()!=0)h_c=panel_c.getHeight();
    sy=1;min_y=Double.MAX_VALUE;max_y=Double.MIN_VALUE;
    for(int i=0; i<valueY.length; i++)valueY[i].removeAllElements();
    offscreen = new BufferedImage(w_c, h_c,BufferedImage.TYPE_3BYTE_BGR);
    offgraphics = (Graphics2D)offscreen.getGraphics();
    getImageGrid();
  }

  public void calc(){
    for(int i=0; i<in.length; i++)if(Double.isInfinite(in[i]))return;
    for(int i=0; i<in.length; i++)out[i]=in[i];
    for(int i=0; i<valueY.length; i++)valueY[i].insertElementAt(new Double(in[i]),0);
    if(valueY[0].size()>w_c){
        for(int i=0; i<in.length; i++){
            valueY[i].setSize(w_c);
            valueY[i].trimToSize();
        }
    }
    for(int i=0; i<in.length; i++){
        min_y=Math.min(min_y,in[i]);
        max_y=Math.max(max_y,in[i]);
    }
    sy=(max_y-min_y)/(h_c-4);
    dy=-min_y/sy+2;
    for(int i=0; i<in.length; i++)in[i]=Double.NaN;
    if(isPainted)return;
    offgraphics.drawImage(offscreen_grid, null, 0, 0);
    offgraphics.setStroke(new BasicStroke(2));
    for(int i=0; i<in.length; i++){
        if(i==0)offgraphics.setColor(Color.RED);
        else if(i==1)offgraphics.setColor(Color.GREEN);
        else if(i==2)offgraphics.setColor(Color.BLUE);
        else if(i==3)offgraphics.setColor(Color.CYAN);
        else if(i==4)offgraphics.setColor(Color.MAGENTA);
        else if(i==5)offgraphics.setColor(Color.ORANGE);
        else if(i==6)offgraphics.setColor(Color.PINK);
        else if(i==7)offgraphics.setColor(Color.WHITE);
        else if(i==8)offgraphics.setColor(Color.YELLOW);
        double[] vy = new double[valueY[i].size()];
        for(int j=0; j<valueY[i].size(); j++){
            vy[j]=h_c-((Double)valueY[i].elementAt(valueY[i].size()-1-j)).doubleValue()/sy-dy;
        }
        try{
            for(int j=0; j<w_c-1; j++) offgraphics.draw(new Line2D.Double(j,vy[j], j+1,vy[j+1]));
        }catch(Exception e) { }
    }
    try{
        offgraphics.setColor(Color.YELLOW);
        offgraphics.drawString(max_y+"", 3, 10);
        offgraphics.drawString(min_y+"", 3, h_c-3);
    }catch(Exception e) { }
    repaint();
  }

  public void calc_post(){
    offgraphics.drawImage(offscreen_grid, null, 0, 0);
    offgraphics.setStroke(new BasicStroke(2));
    for(int i=0; i<in.length; i++){
        if(i==0)offgraphics.setColor(Color.RED);
        else if(i==1)offgraphics.setColor(Color.GREEN);
        else if(i==2)offgraphics.setColor(Color.BLUE);
        else if(i==3)offgraphics.setColor(Color.CYAN);
        else if(i==4)offgraphics.setColor(Color.MAGENTA);
        else if(i==5)offgraphics.setColor(Color.ORANGE);
        else if(i==6)offgraphics.setColor(Color.PINK);
        else if(i==7)offgraphics.setColor(Color.WHITE);
        else if(i==8)offgraphics.setColor(Color.YELLOW);
        double[] vy = new double[valueY[i].size()];
        for(int j=0; j<valueY[i].size(); j++){
            vy[j]=h_c-((Double)valueY[i].elementAt(valueY[i].size()-1-j)).doubleValue()/sy-dy;
        }
        try{
            for(int j=0; j<w_c-1; j++) offgraphics.draw(new Line2D.Double(j,vy[j], j+1,vy[j+1]));
        }catch(Exception e) { }
    }
    try{
        offgraphics.setColor(Color.YELLOW);
        offgraphics.drawString(max_y+"", 3, 10);
        offgraphics.drawString(min_y+"", 3, h_c-3);
    }catch(Exception e) { }
    repaint();
  }

  public void edit_pre(){
    edit_panel = new JPanel(new VerticalFlowLayout());
    tfw.setText(""+w_c);
    tfh.setText(""+h_c);
    list_inputs.setSelectedIndex(in.length-2);
    JPanel p0 = new JPanel(new BorderLayout());
    JPanel p1 = new JPanel(new BorderLayout());
    JPanel p2 = new JPanel(new BorderLayout());
    p0.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Width")),BorderLayout.WEST);
    p0.add(tfw,BorderLayout.CENTER);
    p1.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Height")),BorderLayout.WEST);
    p1.add(tfh,BorderLayout.CENTER);
    p2.add(new JLabel("Inputs"),BorderLayout.WEST);
    p2.add(list_inputs,BorderLayout.CENTER);
    edit_panel.add(p0);
    edit_panel.add(p1);
    edit_panel.add(p2);
  }
  public void edit_post(){
    try {
      in = new double[list_inputs.getSelectedIndex()+2];
      in_text = new String[in.length];
      out = new double[in.length];
      out_text = new String[in.length];
      out_obj = new Object[in.length];
      out_obj_index = new int[in.length];
      for(int i=0; i<in.length; i++){
        in[i] = Double.NaN;
        in_text[i] = "IN"+i;
        out[i] = Double.NaN;
        out_text[i] = "OUT"+i;
      }
      try{ init(); } catch(Exception e2) {error(e2);}
      int hhh = Integer.parseInt(tfh.getText());
      hhh = Math.max(hhh,in.length*(15)+5);
      setSize(Integer.parseInt(tfw.getText())+panel_in.getWidth()*2,hhh);
      validateTree();
      revalidate();
      w_c=panel_c.getWidth();
      h_c=panel_c.getHeight();
      w=w_c;
      h=h_c;
      size_visual_w=w_c;
      size_visual_h=h_c;
      sy=(max_y-min_y)/h_c;
      dy=-min_y/sy;
      offscreen = null;
      validateTree();
    }catch(Exception e) { error(e); }
  }

}
