/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOUSE. See the GNU
 * General Public License for more details.
 *
 * You should have recieved a copy of the GNU General Public License
 * along with this program; if not write to the Free Software
 * Foundation, inc., 59 Temple Place, Suite 330, Boston MA 02111-1307
 * USA
 */


/**
 *
 * @author Yuriy Mikhaylovskiy
 * @email YuriyMikhaylovskiy@yahoo.com
 * 
 */


import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
//import com.sun.image.codec.jpeg.*;

public class JMCADRTCDocument extends JPanel {
  public String doc_name="", doc_autor="", doc_info="";
  private boolean isDebug = false;
  private boolean isVisual = false;
  private String file_name = null;
  private int x=0,y=0;
  private boolean isCalc = false;
  public long time_from = 0, time_to = 0, time_sleep = 10, time_current = 0, time_debug = 500;
  public int window_x = 0, window_y = 0, window_w = 500, window_h = 400;
  public int window_visual_x = 0, window_visual_y = 0, window_visual_w = 0, window_visual_h = 0;
  public boolean window_max = false;
  public boolean window_visual_max = false;
  private Thread tsolve;
  //private Object addObject;

  public JMCADRTCDocument() {
    try {
      this.setLayout(null);
      addMouseListener(new MouseAdapter(){
        public void mouseClicked(MouseEvent e){
          if(e.getClickCount()==2) setConfig();
        }
      });
    } catch(Exception e) { e.printStackTrace(); }
  }

  public void Document_Calc_Stop(){
    isCalc=false;
  }

  public void Document_Calc(){
    if(isCalc){
      System.out.println("*** Interrupted Calc *** "+(file_name==null?"":file_name));
      isCalc=false;
    }else{
      isCalc=true;
      tsolve = new Thread(){
        Component[] comp = getComponents();
        Component[] comp_gen;
        int comp_count = 0;

        public void run(){
          for(int i=0; i<comp.length; i++){
            if(((JMCADObject)comp[i]).isGeneranor) comp_count++;
          }
          comp_gen = new Component[comp_count];
          comp_count = 0;
          for(int i=0; i<comp.length; i++){
            if(((JMCADObject)comp[i]).isGeneranor) {
                comp_gen[comp_count] = comp[i];
                comp_count++;
            }
          }
          time_from = System.currentTimeMillis();
          for(int i=0; i<comp.length; i++) {
             JMCADObject jmcadobj = (JMCADObject)comp[i];
             for(int j=0; j<jmcadobj.in.length; j++)jmcadobj.in[j]=Double.NaN;
             jmcadobj.calc_pre();
          }
          while(isCalc){
            if(time_sleep>0)try{ Thread.sleep(time_sleep); }catch(Exception e) { }
            time_current = System.currentTimeMillis();
            for(int i=0; i<comp_gen.length && isCalc; i++){
              ((JMCADObject)comp_gen[i]).start(time_from,time_current,time_sleep);
            }
          }
          for(int i=0; i<comp.length; i++)  ((JMCADObject)comp[i]).calc_post();
          time_to = System.currentTimeMillis();
          System.out.println("Stop Time:\t"+time_to+" ms ("+(file_name==null?"":file_name)+")");
          System.out.println("Calc Time:\t"+(time_to-time_from)+" ms ("+(file_name==null?"":file_name)+")");
          System.out.println("*** End Calc *** "+(file_name==null?"":file_name));
          isCalc=false;
        }
      };
      tsolve.setPriority(Thread.MIN_PRIORITY);
      tsolve.start();
      System.out.println("*** Start Calc *** "+(file_name==null?"":file_name));
      System.out.println("Start Time:\t"+time_from+" ms ("+(file_name==null?"":file_name)+")");
    }
  }

  public void Document_Clear_Selected(){
    Component[] comp = getComponents();
    for(int j=0; j<comp.length; j++) {
        JMCADObject JMCADO = (JMCADObject)comp[j];
        JMCADO.isSelected = false;
        JMCADO.clear_in();
        JMCADO.clear_out();
        JMCADO.repaint();
    }
    this.repaint();
  }

  public boolean Document_isCalc(){
    return isCalc;
  }
  public boolean Document_isVisual(){
    return isVisual;
  }
  public void Document_setVisual(boolean visual){
    isVisual = visual;
    Component[] comp = getComponents();
    Color cl = new JPanel().getBackground();
    for(int i=0; i<comp.length; i++){
       JMCADObject obj = (JMCADObject)comp[i];
       if(!obj.isVisual) obj.setVisible(!visual);
       else {
           int len = 0;
           if(obj.in.length>0) len+=10;
           if(obj.out.length>0) len+=10;
           obj.panel_in.removeAll();
           obj.panel_out.removeAll();
           obj.panel_in.setBackground(cl);
           obj.panel_out.setBackground(cl);
           obj.panel_c.setBackground(cl);
           Component[] c = obj.panel_c.getComponents();
           for(int j=0; j<c.length; j++) c[j].setBackground(cl);
           c = obj.getComponents();
           for(int j=0; j<c.length; j++) c[j].setBackground(cl);
           obj.setBackground(cl);
           obj.setLocation(obj.position_visual_x, obj.position_visual_y);
           obj.w=obj.size_visual_w;
           obj.h=obj.size_visual_h;
           obj.setSize(obj.size_visual_w+len,obj.size_visual_h);
       }
    }
    this.setBackground(cl);
    Document_Clear_Selected();
  }
  
  
  public static JMCADRTCDocument Document_Open(String st_path) {
    JMCADRTCDocument doc = new JMCADRTCDocument();
    try {
        if(!new File(st_path).exists())return doc;
        RandomAccessFile fin = new RandomAccessFile(st_path,"rw");
        String st,st_obj=null;
        doc.doc_name = "";
        while((st=fin.readLine())!=null){
          if(st.startsWith("[")){
            if(st_obj!=null && st_obj.startsWith("[")){
              StringTokenizer stt = new StringTokenizer(st_obj,"[] =\t");
              String st_Id = stt.nextToken();
              JMCADObject obj = (JMCADObject)Class.forName(stt.nextToken()).newInstance();
              obj.Id = st_Id;
              if(doc.Document_isVisual()) obj.setVisible(obj.isVisual);
              obj.parse(st_obj);
              doc.add(obj);
            }else if(st_obj!=null){
              StringTokenizer stt = new StringTokenizer(st_obj,"\n");
              while(stt.hasMoreTokens()){
                try{
                  String stl = stt.nextToken();
                  stl=stl.trim();
                  if(stl.length()!=0){
                    StringTokenizer stt0 = new StringTokenizer(stl," =\t");
                    String st0 = stt0.nextToken();
                    if(st0.equalsIgnoreCase("TIME_SLEEP")){
                      doc.time_sleep = Integer.parseInt(stt0.nextToken());
                    }else if(st0.equalsIgnoreCase("WINDOW_BOUNDS")){
                      doc.window_x = Integer.parseInt(stt0.nextToken());
                      doc.window_y = Integer.parseInt(stt0.nextToken());
                      doc.window_w = Integer.parseInt(stt0.nextToken());
                      doc.window_h = Integer.parseInt(stt0.nextToken());
                    }else if(st0.equalsIgnoreCase("WINDOW_VISUAL_BOUNDS")){
                      doc.window_visual_x = Integer.parseInt(stt0.nextToken());
                      doc.window_visual_y = Integer.parseInt(stt0.nextToken());
                      doc.window_visual_w = Integer.parseInt(stt0.nextToken());
                      doc.window_visual_h = Integer.parseInt(stt0.nextToken());
                    }else if(st0.equalsIgnoreCase("TIME_DEBUG")){
                      doc.time_debug = Integer.parseInt(stt0.nextToken());
                    }else if(st0.equalsIgnoreCase("WINDOW_MAXIMUM")){
                      doc.window_max = Boolean.parseBoolean(stt0.nextToken());
                    }else if(st0.equalsIgnoreCase("WINDOW_VISUAL_MAXIMUM")){
                      doc.window_visual_max = Boolean.parseBoolean(stt0.nextToken());
                    }else if(st0.equalsIgnoreCase("NAME")){
                      doc.doc_name = "";
                      doc.doc_name = stt0.nextToken("");
                      doc.doc_name = new String(doc.doc_name.substring(doc.doc_name.indexOf("=")+1).trim().getBytes("ISO8859_1"),"Cp1251");
                    }else if(st0.equalsIgnoreCase("AUTOR")){
                      doc.doc_autor = stt0.nextToken("");
                      doc.doc_autor = new String(doc.doc_autor.substring(doc.doc_autor.indexOf("=")+1).trim().getBytes("ISO8859_1"),"Cp1251");
                    }else if(st0.equalsIgnoreCase("INFO")){
                      doc.doc_info = stt0.nextToken("");
                      doc.doc_info = new String(doc.doc_info.substring(doc.doc_info.indexOf("=")+1).trim().getBytes("ISO8859_1"),"Cp1251");
                    }
                  }
                } catch(Exception e1) {return null;}
              }
            }
            st_obj=st;
          } else st_obj+="\n"+st;
        }
        if(st_obj!=null){
          StringTokenizer stt = new StringTokenizer(st_obj,"[] =\t");
          String st_Id = stt.nextToken();
          JMCADObject obj = (JMCADObject)Class.forName(stt.nextToken()).newInstance();
          obj.Id = st_Id;
          if(doc.Document_isVisual()) obj.setVisible(obj.isVisual);
          obj.parse(st_obj);
          doc.add(obj);
        }else if(st_obj!=null){
          StringTokenizer stt = new StringTokenizer(st_obj,"\n");
          while(stt.hasMoreTokens()){
            try{
              String stl = stt.nextToken();
              stl=stl.trim();
              if(stl.length()!=0){
                StringTokenizer stt0 = new StringTokenizer(stl," =\t");
                String st0 = stt0.nextToken();
                if(st0.equalsIgnoreCase("TIME_SLEEP")){
                  doc.time_sleep = Integer.parseInt(stt0.nextToken());
                }else if(st0.equalsIgnoreCase("WINDOW_BOUNDS")){
                  doc.window_x = Integer.parseInt(stt0.nextToken());
                  doc.window_y = Integer.parseInt(stt0.nextToken());
                  doc.window_w = Integer.parseInt(stt0.nextToken());
                  doc.window_h = Integer.parseInt(stt0.nextToken());
                }else if(st0.equalsIgnoreCase("WINDOW_VISUAL_BOUNDS")){
                      doc.window_visual_x = Integer.parseInt(stt0.nextToken());
                      doc.window_visual_y = Integer.parseInt(stt0.nextToken());
                      doc.window_visual_w = Integer.parseInt(stt0.nextToken());
                      doc.window_visual_h = Integer.parseInt(stt0.nextToken());
                }else if(st0.equalsIgnoreCase("TIME_DEBUG")){
                  doc.time_debug = Integer.parseInt(stt0.nextToken());
                }else if(st0.equalsIgnoreCase("WINDOW_MAXIMUM")){
                  doc.window_max = Boolean.parseBoolean(stt0.nextToken());
                }else if(st0.equalsIgnoreCase("NAME")){
                  doc.doc_name = "";
                  doc.doc_name = stt0.nextToken("");
                  doc.doc_name = new String(doc.doc_name.substring(doc.doc_name.indexOf("=")+1).trim().getBytes("ISO8859_1"),"Cp1251");
                }else if(st0.equalsIgnoreCase("AUTOR")){
                  doc.doc_autor = stt0.nextToken("");
                  doc.doc_autor = new String(doc.doc_autor.substring(doc.doc_autor.indexOf("=")+1).trim().getBytes("ISO8859_1"),"Cp1251");
                }else if(st0.equalsIgnoreCase("INFO")){
                  doc.doc_info = stt0.nextToken("");
                  doc.doc_info = new String(doc.doc_info.substring(doc.doc_info.indexOf("=")+1).trim().getBytes("ISO8859_1"),"Cp1251");
                }
              }
            } catch(Exception e1) {return null;}
          }
        }
        Component[] comp = doc.getComponents();
        JMCADObject[] obj = new JMCADObject[comp.length];
        for(int i=0; i<comp.length; i++) obj[i] = (JMCADObject)comp[i];
        for(int i=0; i<obj.length; i++){
          Object[] out_obj = obj[i].out_obj;
          for(int j=0; j<out_obj.length; j++){
            try{
              if(out_obj[j]!=null){
                JMCADObject obj_tmp=null;
                int obj_tmp_index=-1;
                StringTokenizer stt = new StringTokenizer(out_obj[j]+""," ");
                String st_id = stt.nextToken();
                String st_in = stt.nextToken();
                for(int k=0; k<obj.length; k++) if(obj[k].Id.toUpperCase().equals(st_id)) obj_tmp = obj[k];
                String[] in_text = obj_tmp.in_text;
                for(int k=0; k<in_text.length; k++){
                  if(in_text[k].equals(st_in)) obj_tmp_index = k;
                }
                obj[i].out_obj[j]=obj_tmp;
                obj[i].out_obj_index[j]=obj_tmp_index;
              }
            } catch(Exception e1) {return doc;}
          }
        }
        return doc;
    } catch(Exception e1) {return doc;}
  }
  public JMCADRTC getJMCADRTC(){
    Component component = this;
    for(; component != null && !(component instanceof JMCADRTC); component = component.getParent());
    return (JMCADRTC)component;
  }
  
  public void paint(Graphics g){
    super.paint(g);
    if(this.isVisual)return;
    g.setColor(Color.black);
    ((Graphics2D)g).setStroke(new BasicStroke(2));
    Component[] comp = getComponents();
    int y_center = 0;
    for(int i=0; i<comp.length; i++){
      Object[] out_obj = ((JMCADObject)comp[i]).out_obj;
      int[] out_obj_index = ((JMCADObject)comp[i]).out_obj_index;
      for(int j=0; j<out_obj.length; j++){
        try{
          if(out_obj[j]!=null){
            int x_in=0,y_in=0,x_out=0,y_out=0;
            x_out=comp[i].getLocation().x;
            x_out+=((JMCADObject)comp[i]).panel_out.getLocation().x;
            x_out+=((JMCADObject)comp[i]).panel_out.getComponent(j).getLocation().x+((JMCADObject)comp[i]).panel_out.getComponent(j).getWidth()/2;
            y_out=comp[i].getLocation().y;
            y_out+=((JMCADObject)comp[i]).panel_out.getLocation().y;
            y_out+=((JMCADObject)comp[i]).panel_out.getComponent(j).getLocation().y+((JMCADObject)comp[i]).panel_out.getComponent(j).getHeight()/2;
            x_in=((JMCADObject)out_obj[j]).getLocation().x;
            x_in+=((JMCADObject)out_obj[j]).panel_in.getLocation().x;
            x_in+=((JMCADObject)out_obj[j]).panel_in.getComponent(out_obj_index[j]).getLocation().x+((JMCADObject)out_obj[j]).panel_in.getComponent(out_obj_index[j]).getWidth()/2;
            y_in=((JMCADObject)out_obj[j]).getLocation().y;
            y_in+=((JMCADObject)out_obj[j]).panel_in.getLocation().y;
            y_in+=((JMCADObject)out_obj[j]).panel_in.getComponent(out_obj_index[j]).getLocation().y+((JMCADObject)out_obj[j]).panel_in.getComponent(out_obj_index[j]).getHeight()/2;
            if(Math.abs(x_out-x_in)>40 || Math.abs(y_out-y_in)>40){
                g.drawLine(x_out,y_out,x_out+12,y_out);
                if(x_out>x_in){
                    //int y_center = (y_out + y_in)/2;
                    if(y_out < y_in){
                        y_center = (y_out+((JMCADObject)comp[i]).getHeight()/2+y_in-((JMCADObject)out_obj[j]).getHeight()/2)/2;
                    }else {
                        y_center = (y_out-((JMCADObject)comp[i]).getHeight()/2+y_in+((JMCADObject)out_obj[j]).getHeight()/2)/2;
                    }
                    g.drawLine(x_out+12,y_out,x_out+12,y_center);
                    g.drawLine(x_out+12,y_center,x_in-12,y_center);
                    g.drawLine(x_in-12,y_in,x_in-12,y_center);
                }else g.drawLine(x_out+12,y_out,x_in-12,y_in);
                g.drawLine(x_in-12,y_in,x_in,y_in);
            }else g.drawLine(x_in,y_in,x_out,y_out);
          }
        } catch(Exception e) { out_obj[j]=null;}
      }
    }

  }
  public String getFileName(){ return file_name; }
  public void setFileName(String n){ file_name=n; }
  public void error(Object st){
    JOptionPane.showMessageDialog(this,ResourceBundle.getBundle("JMCAD_Internationalize").getString("Error:")+st,ResourceBundle.getBundle("JMCAD_Internationalize").getString("Error!"),JOptionPane.ERROR_MESSAGE);
    if(st instanceof Exception) ((Exception)st).printStackTrace(); else System.out.println(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Error:")+st);
  }
  
  public void setConfig() {
    try{
      JMCADConfigPanel p = new JMCADConfigPanel();
      p.delay.setText(time_sleep+"");
      p.delay_debug.setText(time_debug+"");
      p.delay_debug.setEditable(false);
      p.name.setText(doc_name);
      p.autor.setText(doc_autor);
      p.info.setText(doc_info);
      Object[] b = {ResourceBundle.getBundle("JMCAD_Internationalize").getString("Ok"),ResourceBundle.getBundle("JMCAD_Internationalize").getString("Cancel")};
      int res = JOptionPane.showOptionDialog(this,p,ResourceBundle.getBundle("JMCAD_Internationalize").getString("Configuring_model"),JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE,null,b,b[0]);
      if(res==0){
        time_sleep=Integer.parseInt(p.delay.getText());
        doc_name=p.name.getText();
        doc_autor=p.autor.getText();
        doc_info=p.info.getText();
        time_debug=Integer.parseInt(p.delay_debug.getText());
      }
    }catch(Exception e1) { error(e1); }
  }
  
  
}
