/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOUSE. See the GNU
 * General Public License for more details.
 *
 * You should have recieved a copy of the GNU General Public License
 * along with this program; if not write to the Free Software
 * Foundation, inc., 59 Temple Place, Suite 330, Boston MA 02111-1307
 * USA
 */


/**
 *
 * @author Yuriy Mikhaylovskiy
 * @email YuriyMikhaylovskiy@yahoo.com
 * 
 */


import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.*;

public class JMCADRTSDocument extends JPanel {
  public String doc_name="", doc_autor="", doc_info="";
  private String file_name = null;
  private boolean isCalc = false;
  public long time_from = 0, time_to = 0, time_sleep = 10, time_current = 0, time_debug = 500;
  public int window_x = 0, window_y = 0, window_w = 500, window_h = 400;
  private Thread tsolve;
  
  public JMCADRTSDocument() {
    try {
      this.setLayout(null);
      this.setBackground(Color.white);
      addMouseListener(new MouseAdapter(){
        public void mouseClicked(MouseEvent e){
          if(e.getClickCount()==2) setConfig();
        }
      });      
    } catch(Exception e) { e.printStackTrace(); }
  }

  
  public void Document_Calc_Stop(){
    isCalc=false;
  }

  public void Document_Calc(){
    if(isCalc){
      System.out.println("*** Interrupted Calc *** "+(file_name==null?"":file_name));
      isCalc=false;
    }else{
      isCalc=true;
      tsolve = new Thread(){
        Component[] comp = getComponents();
        Component[] comp_gen;
        int comp_count = 0;

        public void run(){
          for(int i=0; i<comp.length; i++){
            if(((JMCADObject)comp[i]).isGeneranor) comp_count++;
          }
          comp_gen = new Component[comp_count];
          comp_count = 0;
          for(int i=0; i<comp.length; i++){
            if(((JMCADObject)comp[i]).isGeneranor) {
                comp_gen[comp_count] = comp[i];
                comp_count++;
            }
          }
          time_from = System.currentTimeMillis();
          for(int i=0; i<comp.length; i++)  ((JMCADObject)comp[i]).calc_pre();
          while(isCalc){
            if(time_sleep>0)try{ Thread.sleep(time_sleep); }catch(Exception e) { }
            time_current = System.currentTimeMillis();
            for(int i=0; i<comp_gen.length && isCalc; i++){
              ((JMCADObject)comp_gen[i]).start(time_from,time_current,time_sleep);              
            }            
          }
          for(int i=0; i<comp.length; i++)  ((JMCADObject)comp[i]).calc_post();
          time_to = System.currentTimeMillis();
          System.out.println("Stop Time:\t"+time_to+" ms ("+(file_name==null?"":file_name)+")");
          System.out.println("Calc Time:\t"+(time_to-time_from)+" ms ("+(file_name==null?"":file_name)+")");
          System.out.println("*** End Calc *** "+(file_name==null?"":file_name));
          isCalc=false;
        }
      };
      tsolve.setPriority(Thread.MIN_PRIORITY);
      tsolve.start();
      System.out.println("*** Start Calc *** "+(file_name==null?"":file_name));
      System.out.println("Start Time:\t"+time_from+" ms ("+(file_name==null?"":file_name)+")");
    }
  }

  
  public boolean Document_isCalc(){
    return isCalc;
  }
  
  
  public JMCADRTS getJMCADRTS(){
    Component component = this;
    for(; component != null && !(component instanceof JMCADRTS); component = component.getParent());
    return (JMCADRTS)component;
  }

  public void paint(Graphics g){
    super.paint(g);
    g.setColor(Color.black);
    ((Graphics2D)g).setStroke(new BasicStroke(2));
    Component[] comp = getComponents();
    int y_center = 0;
    for(int i=0; i<comp.length; i++){
      Object[] out_obj = ((JMCADObject)comp[i]).out_obj;
      int[] out_obj_index = ((JMCADObject)comp[i]).out_obj_index;
      for(int j=0; j<out_obj.length; j++){
        try{
          if(out_obj[j]!=null){
            int x_in=0,y_in=0,x_out=0,y_out=0;
            x_out=comp[i].getLocation().x;
            x_out+=((JMCADObject)comp[i]).panel_out.getLocation().x;
            x_out+=((JMCADObject)comp[i]).panel_out.getComponent(j).getLocation().x+((JMCADObject)comp[i]).panel_out.getComponent(j).getWidth()/2;
            y_out=comp[i].getLocation().y;
            y_out+=((JMCADObject)comp[i]).panel_out.getLocation().y;
            y_out+=((JMCADObject)comp[i]).panel_out.getComponent(j).getLocation().y+((JMCADObject)comp[i]).panel_out.getComponent(j).getHeight()/2;
            x_in=((JMCADObject)out_obj[j]).getLocation().x;
            x_in+=((JMCADObject)out_obj[j]).panel_in.getLocation().x;
            x_in+=((JMCADObject)out_obj[j]).panel_in.getComponent(out_obj_index[j]).getLocation().x+((JMCADObject)out_obj[j]).panel_in.getComponent(out_obj_index[j]).getWidth()/2;
            y_in=((JMCADObject)out_obj[j]).getLocation().y;
            y_in+=((JMCADObject)out_obj[j]).panel_in.getLocation().y;
            y_in+=((JMCADObject)out_obj[j]).panel_in.getComponent(out_obj_index[j]).getLocation().y+((JMCADObject)out_obj[j]).panel_in.getComponent(out_obj_index[j]).getHeight()/2;
            if(Math.abs(x_out-x_in)>40 || Math.abs(y_out-y_in)>40){
                g.drawLine(x_out,y_out,x_out+12,y_out);
                if(x_out>x_in){
                    if(y_out < y_in){
                        y_center = (y_out+((JMCADObject)comp[i]).getHeight()/2+y_in-((JMCADObject)out_obj[j]).getHeight()/2)/2;
                    }else {
                        y_center = (y_out-((JMCADObject)comp[i]).getHeight()/2+y_in+((JMCADObject)out_obj[j]).getHeight()/2)/2;
                    }
                    g.drawLine(x_out+12,y_out,x_out+12,y_center);
                    g.drawLine(x_out+12,y_center,x_in-12,y_center);
                    g.drawLine(x_in-12,y_in,x_in-12,y_center);
                }else g.drawLine(x_out+12,y_out,x_in-12,y_in);
                g.drawLine(x_in-12,y_in,x_in,y_in);
            }else g.drawLine(x_in,y_in,x_out,y_out);
          }
        } catch(Exception e) { out_obj[j]=null;}
      }
    }

  }

  public void error(Object st){
    JOptionPane.showMessageDialog(this,ResourceBundle.getBundle("JMCAD_Internationalize").getString("Error:")+st,ResourceBundle.getBundle("JMCAD_Internationalize").getString("Error!"),JOptionPane.ERROR_MESSAGE);
    if(st instanceof Exception) ((Exception)st).printStackTrace(); else System.out.println(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Error:")+st);
  }
  
  public void setConfig() {
    try{
      JMCADConfigPanel p = new JMCADConfigPanel();
      p.delay.setText(time_sleep+"");
      p.delay_debug.setText(time_debug+"");
      p.delay_debug.setEditable(false);
      p.name.setText(doc_name);
      p.autor.setText(doc_autor);
      p.info.setText(doc_info);
      Object[] b = {ResourceBundle.getBundle("JMCAD_Internationalize").getString("Ok"),ResourceBundle.getBundle("JMCAD_Internationalize").getString("Cancel")};
      int res = JOptionPane.showOptionDialog(this,p,ResourceBundle.getBundle("JMCAD_Internationalize").getString("Configuring_model"),JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE,null,b,b[0]);
      if(res==0){
        time_sleep=Integer.parseInt(p.delay.getText());
        doc_name=p.name.getText();
        doc_autor=p.autor.getText();
        doc_info=p.info.getText();
        time_debug=Integer.parseInt(p.delay_debug.getText());
      }
    }catch(Exception e1) { error(e1); }
  }
  
  
}
